@@ -1,8 +1,8 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE OverloadedStrings #-}
 module Main where
 
 import Control.Monad (void)
+import Control.Monad.State (modify)
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid ((<>))
 #endif
@@ -16,7 +16,8 @@ import Brick.Types
   , BrickEvent(..)
   )
 import Brick.Widgets.Core
-  ( vBox
+  ( Padding(..)
+  , vBox
   , padTopBottom
   , withDefAttr
   , cached
@@ -29,6 +30,7 @@ import Brick.Widgets.Center
   )
 import Brick.AttrMap
   ( AttrName
+  , attrName
   , attrMap
   )
 
@@ -50,26 +52,26 @@ drawUi i = [ui]
              , padTopBottom 1 $
                cached ExpensiveWidget $
                withDefAttr emphAttr $ str $ "This widget is cached (state = " <> show i <> ")"
-             , padBottom (T.Pad 1) $
+             , padBottom (Pad 1) $
                withDefAttr emphAttr $ str $ "This widget is not cached (state = " <> show i <> ")"
              , hCenter $ str "Press 'i' to invalidate the cache,"
              , str "'+' to change the state value, and"
              , str "'Esc' to quit."
              ]
 
-appEvent :: Int -> BrickEvent Name e -> T.EventM Name (T.Next Int)
-appEvent i (VtyEvent (V.EvKey (V.KChar '+') [])) = M.continue $ i + 1
-appEvent i (VtyEvent (V.EvKey (V.KChar 'i') [])) = M.invalidateCacheEntry ExpensiveWidget >> M.continue i
-appEvent i (VtyEvent (V.EvKey V.KEsc [])) = M.halt i
-appEvent i _ = M.continue i
+appEvent :: BrickEvent Name e -> T.EventM Name Int ()
+appEvent (VtyEvent (V.EvKey (V.KChar '+') [])) = modify (+ 1)
+appEvent (VtyEvent (V.EvKey (V.KChar 'i') [])) = M.invalidateCacheEntry ExpensiveWidget
+appEvent (VtyEvent (V.EvKey V.KEsc [])) = M.halt
+appEvent _ = return ()
 
 emphAttr :: AttrName
-emphAttr = "emphasis"
+emphAttr = attrName "emphasis"
 
 app :: M.App Int e Name
 app =
     M.App { M.appDraw = drawUi
-          , M.appStartEvent = return
+          , M.appStartEvent = return ()
           , M.appHandleEvent = appEvent
           , M.appAttrMap = const $ attrMap V.defAttr [(emphAttr, V.white `on` V.blue)]
           , M.appChooseCursor = M.neverShowCursor
