@@ -2,7 +2,7 @@ module Main where
 
 import Brick
 
-ui :: Widget
+ui :: Widget ()
 ui = str "Hello, world!"
 
 main :: IO ()
