@@ -1,11 +1,12 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
 module Main where
 
-import Lens.Micro ((^.), (&), (.~), (%~))
+import Lens.Micro ((^.))
 import Lens.Micro.TH (makeLenses)
+import Lens.Micro.Mtl
 import Control.Monad (void)
+import Control.Monad.Trans (liftIO)
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid ((<>))
 #endif
@@ -20,8 +21,6 @@ import qualified Brick.Widgets.Edit as E
 import qualified Brick.Widgets.Center as C
 import qualified Brick.Widgets.Border as B
 import Brick.Widgets.Core
-import Data.Text.Zipper (moveCursor)
-import Data.Tuple (swap)
 
 data Name = Info | Button1 | Button2 | Button3 | Prose | TextBox
           deriving (Show, Ord, Eq)
@@ -45,15 +44,15 @@ drawUi st =
 buttonLayer :: St -> Widget Name
 buttonLayer st =
     C.vCenterLayer $
-      C.hCenterLayer (padBottom (T.Pad 1) $ str "Click a button:") <=>
+      C.hCenterLayer (padBottom (Pad 1) $ str "Click a button:") <=>
       C.hCenterLayer (hBox $ padLeftRight 1 <$> buttons) <=>
       C.hCenterLayer (padTopBottom 1 $ str "Or enter text and then click in this editor:") <=>
       C.hCenterLayer (vLimit 3 $ hLimit 50 $ E.renderEditor (str . unlines) True (st^.edit))
     where
         buttons = mkButton <$> buttonData
-        buttonData = [ (Button1, "Button 1", "button1")
-                     , (Button2, "Button 2", "button2")
-                     , (Button3, "Button 3", "button3")
+        buttonData = [ (Button1, "Button 1", attrName "button1")
+                     , (Button2, "Button 2", attrName "button2")
+                     , (Button3, "Button 3", attrName "button3")
                      ]
         mkButton (name, label, attr) =
             let wasClicked = (fst <$> st^.lastReportedClick) == Just name
@@ -69,11 +68,7 @@ proseLayer st =
   B.border $
   C.hCenterLayer $
   vLimit 8 $
-  -- n.b. if clickable and viewport are inverted here, click event
-  -- coordinates will only identify the viewable range, not the actual
-  -- editor widget coordinates.
   viewport Prose Vertical $
-  clickable Prose $
   vBox $ map str $ lines (st^.prose)
 
 infoLayer :: St -> Widget Name
@@ -81,38 +76,46 @@ infoLayer st = T.Widget T.Fixed T.Fixed $ do
     c <- T.getContext
     let h = c^.T.availHeightL
         msg = case st^.lastReportedClick of
-                Nothing -> "nothing"
-                Just (name, T.Location l) -> show name <> " at " <> show l
+                Nothing ->
+                    "Click and hold/drag to report a mouse click"
+                Just (name, T.Location l) ->
+                    "Mouse down at " <> show name <> " @ " <> show l
     T.render $ translateBy (T.Location (0, h-1)) $ clickable Info $
-               withDefAttr "info" $
-               C.hCenter (str ("Last reported click: " <> msg))
+               withDefAttr (attrName "info") $
+               C.hCenter $ str msg
 
-appEvent :: St -> T.BrickEvent Name e -> T.EventM Name (T.Next St)
-appEvent st (T.MouseDown n _ _ loc) = do
-    let T.Location pos = loc
-    M.continue $ st & lastReportedClick .~ Just (n, loc)
-                    & edit %~ E.applyEdit (if n == TextBox then moveCursor (swap pos) else id)
-appEvent st (T.MouseUp {}) = M.continue $ st & lastReportedClick .~ Nothing
-appEvent st (T.VtyEvent (V.EvMouseUp {})) = M.continue $ st & lastReportedClick .~ Nothing
-appEvent st (T.VtyEvent (V.EvKey V.KUp [V.MCtrl])) = M.vScrollBy (M.viewportScroll Prose) (-1) >> M.continue st
-appEvent st (T.VtyEvent (V.EvKey V.KDown [V.MCtrl])) = M.vScrollBy (M.viewportScroll Prose) 1 >> M.continue st
-appEvent st (T.VtyEvent (V.EvKey V.KEsc [])) = M.halt st
-appEvent st (T.VtyEvent ev) = M.continue =<< T.handleEventLensed st edit E.handleEditorEvent ev
-appEvent st _ = M.continue st
+appEvent :: T.BrickEvent Name e -> T.EventM Name St ()
+appEvent ev@(T.MouseDown n _ _ loc) = do
+    lastReportedClick .= Just (n, loc)
+    zoom edit $ E.handleEditorEvent ev
+appEvent (T.MouseUp {}) =
+    lastReportedClick .= Nothing
+appEvent (T.VtyEvent (V.EvMouseUp {})) =
+    lastReportedClick .= Nothing
+appEvent (T.VtyEvent (V.EvKey V.KUp [V.MCtrl])) =
+    M.vScrollBy (M.viewportScroll Prose) (-1)
+appEvent (T.VtyEvent (V.EvKey V.KDown [V.MCtrl])) =
+    M.vScrollBy (M.viewportScroll Prose) 1
+appEvent (T.VtyEvent (V.EvKey V.KEsc [])) =
+    M.halt
+appEvent ev =
+    zoom edit $ E.handleEditorEvent ev
 
 aMap :: AttrMap
 aMap = attrMap V.defAttr
-    [ ("info",      V.white `on` V.magenta)
-    , ("button1",   V.white `on` V.cyan)
-    , ("button2",   V.white `on` V.green)
-    , ("button3",   V.white `on` V.blue)
+    [ (attrName "info",      V.white `on` V.magenta)
+    , (attrName "button1",   V.white `on` V.cyan)
+    , (attrName "button2",   V.white `on` V.green)
+    , (attrName "button3",   V.white `on` V.blue)
     , (E.editFocusedAttr, V.black `on` V.yellow)
     ]
 
 app :: M.App St e Name
 app =
     M.App { M.appDraw = drawUi
-          , M.appStartEvent = return
+          , M.appStartEvent = do
+              vty <- M.getVtyHandle
+              liftIO $ V.setMode (V.outputIface vty) V.Mouse True
           , M.appHandleEvent = appEvent
           , M.appAttrMap = const aMap
           , M.appChooseCursor = M.showFirstCursor
@@ -120,13 +123,7 @@ app =
 
 main :: IO ()
 main = do
-    let buildVty = do
-          v <- V.mkVty =<< V.standardIOConfig
-          V.setMode (V.outputIface v) V.Mouse True
-          return v
-
-    initialVty <- buildVty
-    void $ M.customMain initialVty buildVty Nothing app $ St [] Nothing
+    void $ M.defaultMain app $ St [] Nothing
            (unlines [ "Try clicking on various UI elements."
                     , "Observe that the click coordinates identify the"
                     , "underlying widget coordinates."
