@@ -25,6 +25,7 @@ module Brick.Widgets.Edit
   , editorText
   -- * Reading editor contents
   , getEditContents
+  , getCursorPosition
   -- * Handling events
   , handleEditorEvent
   -- * Editing text
@@ -53,6 +54,7 @@ import qualified Data.Text.Encoding as T
 import qualified Data.Text.Zipper as Z hiding ( textZipper )
 import qualified Data.Text.Zipper.Generic as Z
 import qualified Data.Text.Zipper.Generic.Words as Z
+import Data.Tuple (swap)
 
 import Brick.Types
 import Brick.Widgets.Core
@@ -60,6 +62,7 @@ import Brick.AttrMap
 
 -- | Editor state.  Editors support the following events by default:
 --
+-- * Mouse clicks: change cursor position
 -- * Meta-<: go to beginning of file
 -- * Meta->: go to end of file
 -- * Ctrl-a, Home: go to beginning of line
@@ -114,48 +117,56 @@ instance DecodeUtf8 T.Text where
 instance DecodeUtf8 String where
     decodeUtf8 bs = T.unpack <$> decodeUtf8 bs
 
-handleEditorEvent :: (DecodeUtf8 t, Eq t, Z.GenericTextZipper t)
-                  => Event -> Editor t n -> EventM n (Editor t n)
-handleEditorEvent e ed =
-        let f = case e of
-                  EvPaste bs -> case decodeUtf8 bs of
-                      Left _ -> id
-                      Right t -> Z.insertMany t
-                  EvKey (KChar 'a') [MCtrl] -> Z.gotoBOL
-                  EvKey (KChar 'e') [MCtrl] -> Z.gotoEOL
-                  EvKey (KChar 'd') [MCtrl] -> Z.deleteChar
-                  EvKey (KChar 'd') [MMeta] -> Z.deleteWord
-                  EvKey (KChar 'k') [MCtrl] -> Z.killToEOL
-                  EvKey (KChar 'u') [MCtrl] -> Z.killToBOL
-                  EvKey KEnter [] -> Z.breakLine
-                  EvKey KDel [] -> Z.deleteChar
-                  EvKey (KChar c) [] | c /= '\t' -> Z.insertChar c
-                  EvKey KUp [] -> Z.moveUp
-                  EvKey KDown [] -> Z.moveDown
-                  EvKey KLeft [] -> Z.moveLeft
-                  EvKey KRight [] -> Z.moveRight
-                  EvKey (KChar 'b') [MCtrl] -> Z.moveLeft
-                  EvKey (KChar 'f') [MCtrl] -> Z.moveRight
-                  EvKey (KChar 'b') [MMeta] -> Z.moveWordLeft
-                  EvKey (KChar 'f') [MMeta] -> Z.moveWordRight
-                  EvKey KBS [] -> Z.deletePrevChar
-                  EvKey (KChar 't') [MCtrl] -> Z.transposeChars
-                  EvKey KHome [] -> Z.gotoBOL
-                  EvKey KEnd [] -> Z.gotoEOL
-                  EvKey (KChar '<') [MMeta] -> Z.gotoBOF
-                  EvKey (KChar '>') [MMeta] -> Z.gotoEOF
-                  _ -> id
-        in return $ applyEdit f ed
+handleEditorEvent :: (Eq n, DecodeUtf8 t, Eq t, Z.GenericTextZipper t)
+                  => BrickEvent n e
+                  -> EventM n (Editor t n) ()
+handleEditorEvent e = do
+    ed <- get
+    let f = case e of
+              VtyEvent ev ->
+                  handleVtyEvent ev
+              MouseDown n _ _ (Location pos) | n == getName ed ->
+                  Z.moveCursorClosest (swap pos)
+              _ -> id
+        handleVtyEvent ev = case ev of
+            EvPaste bs -> case decodeUtf8 bs of
+                Left _ -> id
+                Right t -> Z.insertMany t
+            EvKey (KChar 'a') [MCtrl] -> Z.gotoBOL
+            EvKey (KChar 'e') [MCtrl] -> Z.gotoEOL
+            EvKey (KChar 'd') [MCtrl] -> Z.deleteChar
+            EvKey (KChar 'd') [MMeta] -> Z.deleteWord
+            EvKey (KChar 'k') [MCtrl] -> Z.killToEOL
+            EvKey (KChar 'u') [MCtrl] -> Z.killToBOL
+            EvKey KEnter [] -> Z.breakLine
+            EvKey KDel [] -> Z.deleteChar
+            EvKey (KChar c) [] | c /= '\t' -> Z.insertChar c
+            EvKey KUp [] -> Z.moveUp
+            EvKey KDown [] -> Z.moveDown
+            EvKey KLeft [] -> Z.moveLeft
+            EvKey KRight [] -> Z.moveRight
+            EvKey (KChar 'b') [MCtrl] -> Z.moveLeft
+            EvKey (KChar 'f') [MCtrl] -> Z.moveRight
+            EvKey (KChar 'b') [MMeta] -> Z.moveWordLeft
+            EvKey (KChar 'f') [MMeta] -> Z.moveWordRight
+            EvKey KBS [] -> Z.deletePrevChar
+            EvKey (KChar 't') [MCtrl] -> Z.transposeChars
+            EvKey KHome [] -> Z.gotoBOL
+            EvKey KEnd [] -> Z.gotoEOL
+            EvKey (KChar '<') [MMeta] -> Z.gotoBOF
+            EvKey (KChar '>') [MMeta] -> Z.gotoEOF
+            _ -> id
+    put $ applyEdit f ed
 
 -- | Construct an editor over 'Text' values
 editorText :: n
-       -- ^ The editor's name (must be unique)
-       -> Maybe Int
-       -- ^ The limit on the number of lines in the editor ('Nothing'
-       -- means no limit)
-       -> T.Text
-       -- ^ The initial content
-       -> Editor T.Text n
+           -- ^ The editor's name (must be unique)
+           -> Maybe Int
+           -- ^ The limit on the number of lines in the editor ('Nothing'
+           -- means no limit)
+           -> T.Text
+           -- ^ The initial content
+           -> Editor T.Text n
 editorText = editor
 
 -- | Construct an editor over 'String' values
@@ -170,29 +181,35 @@ editor :: Z.GenericTextZipper a
        -> Editor a n
 editor name limit s = Editor (Z.textZipper (Z.lines s) limit) name
 
--- | Apply an editing operation to the editor's contents. Bear in mind
--- that you should only apply zipper operations that operate on the
--- current line; the editor will only ever render the first line of
--- text.
+-- | Apply an editing operation to the editor's contents.
+--
+-- This is subject to the restrictions of the underlying text zipper;
+-- for example, if the underlying zipper has a line limit configured,
+-- any edits applied here will be ignored if they edit text outside
+-- the line limit.
 applyEdit :: (Z.TextZipper t -> Z.TextZipper t)
-          -- ^ The 'Data.Text.Zipper' editing transformation to apply
+          -- ^ The 'Z.TextZipper' editing transformation to apply
           -> Editor t n
           -> Editor t n
 applyEdit f e = e & editContentsL %~ f
 
 -- | The attribute assigned to the editor when it does not have focus.
 editAttr :: AttrName
-editAttr = "edit"
+editAttr = attrName "edit"
 
 -- | The attribute assigned to the editor when it has focus. Extends
 -- 'editAttr'.
 editFocusedAttr :: AttrName
-editFocusedAttr = editAttr <> "focused"
+editFocusedAttr = editAttr <> attrName "focused"
 
 -- | Get the contents of the editor.
 getEditContents :: Monoid t => Editor t n -> [t]
 getEditContents e = Z.getText $ e^.editContentsL
 
+-- | Get the cursor position of the editor (row, column).
+getCursorPosition :: Editor t n -> (Int, Int)
+getCursorPosition e = Z.cursorPosition $ e^.editContentsL
+
 -- | Turn an editor state value into a widget. This uses the editor's
 -- name for its scrollable viewport handle and the name is also used to
 -- report mouse events.
